<?php

class OrderController extends OrderControllerCore
{
    public function initContent()
    {
        if (Tools::getValue('rebuildOrder')) {
            $this->skipCheckoutFormSteps();
        }
        parent::initContent();
    }

    public function postProcess()
    {
        if (Tools::isSubmit('submitReorder')
            && $id_order = (int) Tools::getValue('id_order')
        ) {
            $oldCart = new Cart(Order::getCartIdStatic($id_order, $this->context->customer->id));
            $duplication = $oldCart->duplicate();
            if (!$duplication || !Validate::isLoadedObject($duplication['cart'])) {
                $this->errors[] = $this->trans('Sorry. We cannot renew your order.', [], 'Shop.Notifications.Error');
            } elseif (!$duplication['success']) {
                $this->errors[] = $this->trans(
                    'Some items are no longer available, and we are unable to renew your order.',
                    [],
                    'Shop.Notifications.Error'
                );
            } else {
                $this->context->cookie->id_cart = $duplication['cart']->id;
                $context = $this->context;
                $context->cart = $duplication['cart'];
                CartRule::autoAddToCart($context);
                $this->context->cookie->write();
                Tools::redirect('index.php?controller=order&rebuildOrder=true');
            }
        }

        $this->bootstrap();
    }

    protected function bootstrap()
    {
        $translator = $this->getTranslator();
        $session = $this->getCheckoutSession();
        $this->checkoutProcess = $this->buildCheckoutProcess($session, $translator);

        Hook::exec('actionCheckoutRender', ['checkoutProcess' => &$this->checkoutProcess]);
    }

    // walk through each step, and submit same data from previous cart
    private function skipCheckoutFormSteps()
    {
        $customer = new Customer($this->context->cookie->id_customer);

        foreach ($this->checkoutProcess->getSteps() as $step) {
            if ($step instanceof CheckoutPersonalInformationStepCore && !$step->isComplete()) {
                $request = array(
                    'id_gender' => $customer->id_gender,
                    'firstname' => $customer->firstname,
                    'lastname' => $customer->lastname,
                    'email' => $customer->email,
                    'password' => '',
                    'birthday' => ($customer->birthday !== '0000-00-00') ? $customer->birthday : '',
                    'customer_privacy' => 1,
                    'psgdpr' => 1,
                    'submitCreate' => 1,
                    'continue' => 1
                );
                $step->handleRequest($request);
                $this->saveDataToPersist($this->checkoutProcess);
                $this->checkoutProcess
                    ->setNextStepReachable()
                    ->markCurrentStep();
            }

            if ($step instanceof CheckoutAddressesStepCore && !$step->isComplete()) {
                // load old cart for data
                $oldCart = new Cart($this->context->cookie->id_cart);

                $existingAddress = array(
                    'id_address_delivery' => $oldCart->id_address_delivery,
                    'confirm-addresses' => 1,
                    'use_same_address' => 1,
                    'submitAddress' => 1,
                );

                $step->handleRequest($existingAddress);
                // current step has to be set false else wont move on
                $step->setCurrent(false);
                $this->checkoutProcess
                    ->setNextStepReachable()
                    ->markCurrentStep();
                $this->saveDataToPersist($this->checkoutProcess);
            }

            if ($step instanceof CheckoutDeliveryStepCore && !$step->isComplete()) {
                $deliveryOption = json_decode($oldCart->delivery_option, true);
                $arrayKeys = array_keys($deliveryOption);
                $key = reset($arrayKeys);
                $request = array(
                    "delivery_option[$key]" => $deliveryOption[$key],
                    "delivery_message" => Message::getMessageByCartId($this->context->cart->id),
                    "confirmDeliveryOption" => 1,
                );

                $step->handleRequest($request);
                // current step has to be set false else wont move on
                $step->setCurrent(false);
                $this->checkoutProcess
                    ->setNextStepReachable()
                    ->markCurrentStep();
                $this->saveDataToPersist($this->checkoutProcess);
            }
        }
    }
}